@@ -127,15 +127,16 @@
         (cons event traces)))
     (let ((labels (labels pc)))
       (map (cute event->label-traces pc <>) labels)))
-
   (cond
    ((is-a? (%sut) <runtime:port>)
     (event-traces-alist pc))
    ((is-a? (%sut) <runtime:component>)
     (let* ((interface (interfaces-event-traces-alist pc))
            (interface-eligible (eligible-labels interface))
+           (external-eligible (map trigger->string
+                                   (append-map cdr (.external-q pc))))
            (component (event-traces-alist pc)))
-      (filter (compose (cute member <> interface-eligible) car)
+      (filter (compose (cute member <> (append interface-eligible external-eligible)) car)
               component)))
    ((is-a? (%sut) <runtime:system>)
     (interfaces-event-traces-alist pc))))
@@ -478,6 +479,7 @@ final REPORT and return exit status."
       (or (and deadlock-traces
                (pair? (.blocked pc))
                (report traces
+                       #:eligible #f
                        #:internal? internal?
                        #:locations? locations?
                        #:trace trace
@@ -751,7 +753,7 @@ status."
     (when (equal? trace "trace")
       (serialize-header (.state pc) (current-output-port))
       (newline))
-    (or (report (list (list pc)) #:trace trace)
+    (or (report (list (list pc)) #:trace trace #:eligible #f)
         (match (check-invariants (list pc))
           ((traces ..1) (report traces #:trace trace))
           (_ #f))
@@ -828,6 +830,7 @@ status."
                        (loop blocked))
                       ((pair? non-blocked)
                        (or (report non-blocked
+                                   #:eligible #f
                                    #:internal? internal?
                                    #:locations? locations?
                                    #:state? state?
