@@ -1,10 +1,10 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2016, 2018, 2019, 2020, 2021, 2022, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2016, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Henk Katerberg <hank@mudball.nl>
 ;;; Copyright © 2018, 2021, 2022, 2023, 2024, 2025 Rutger (regtur) van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2018, 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2018, 2020, 2021, 2022, 2023, 2024, 2025 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2017, 2018 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;;
 ;;; This file is part of Dezyne.
@@ -33,7 +33,7 @@
   #:use-module (ice-9 poe)
   #:use-module (ice-9 rdelim)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast)
   #:use-module (dzn code)
   #:use-module (dzn code language makreel)
@@ -227,9 +227,6 @@ actions."
       (newline)
       (display (makreel:init-process (options-init options))))))
 
-(define (component-stage)
-  "verify-component")
-
 (define (in-out:dzn->aut+provides-aut options)
   (let* ((model (options-model options))
          (root (options-root options))
@@ -240,7 +237,7 @@ actions."
            ;; "aut-failures" pipeline.  Running the already memoized
            ;; "verify-component" gives us the same result, and has
            ;; already been memoized.
-           (get-lts (result-split (verify-pipeline (component-stage) root model)))
+           (get-lts (result-split (verify-pipeline "verify-component" root model)))
            "\n\x04\n"
            (verify-pipeline "aut-failures" root model #:init provides-init)))))
 
@@ -270,8 +267,11 @@ actions."
     `("lpsparelm" ,(if timings? "--timings" "--quiet"))))
 
 (define (in-out:lps->aut options)
-  (let ((timings? (dzn:command-line:get 'timings)))
-    `("lps2lts" ,(if timings? "--timings" "--quiet")
+  (let ((timings? (dzn:command-line:get 'timings))
+        (threads (dzn:command-line:get 'threads)))
+    `("lps2lts"
+      ,@(if threads `(,(simple-format #f "--threads=~a" threads)) '())
+      ,(if timings? "--timings" "--quiet")
       "--cached" "--out=aut""--save-at-end" "-" "-")))
 
 (define (in-out:aut->aut-weak-trace options)
@@ -584,6 +584,7 @@ init for MODEL unless INIT."
                    (let ((keep-error?
                           (member last '(<missing-reply>
                                          <second-reply>
+                                         <invariant>
                                          <range-error>
                                          <type-error>))))
                      (if keep-error? last
@@ -611,6 +612,7 @@ init for MODEL unless INIT."
                     ((<type-error>) (format #f "type error in model ~a" model-name))
                     ((<missing-reply>) (format #f "reply missing from model ~a" model-name))
                     ((<second-reply>) (format #f "double reply in model ~a" model-name))
+                    ((<invariant>) (format #f "invariant violation in model ~a" model-name))
                     ((<queue-full>) (format #f "queue full in model ~a" model-name))
                     (else (format #f "~a in model ~a" error model-name))))
          (trace (remove-flushes trace))
@@ -637,7 +639,7 @@ init for MODEL unless INIT."
               model-name assert))
     #f))
 
-(define (report assert skip? trace model)
+(define* (report model assert trace #:key skip?)
   (cond (skip? (report-skip model assert))
         (trace (report-fail model assert trace))
         (else  (report-ok   model assert))))
@@ -647,13 +649,12 @@ init for MODEL unless INIT."
 ;;; Verify model.
 ;;;
 
-(define (reduce-or all? l)
-  (if all? (fold (cut or <> <>) #f (map (cute <>) l))
+(define (reduce-or keep-going? l)
+  (if keep-going? (fold (cut or <> <>) #f (map (cute <>) l))
       (fold (lambda (e res) (or res (e))) #f l)))
 
-(define (mcrl2:verify-compliance root model)
-  (let* ((output status (verify-pipeline "verify-compliance" root model))
-         (lines (and output (string-split output #\newline)))
+(define (mcrl2:compliance-output->trace output status)
+  (let* ((lines (and output (string-split output #\newline)))
          (stdout-status (and lines
                              (filter (cute string-prefix? "result: " <>) lines)))
          (stdout-status (and (pair? stdout-status) (car stdout-status)))
@@ -695,69 +696,125 @@ init for MODEL unless INIT."
       (throw 'programming-error (format #f "status: ~s, trace: ~s\n" status trace)))
     (values trace interface-accepts component-accepts)))
 
-(define (mcrl2:verify-interface-asserts model root)
-  (let* ((model-name (makreel:unticked-dotted-name model))
-         (asserts (verify-pipeline "verify-interface" root model))
-         (asserts (result-split asserts))
-         (nondets (verify-pipeline "verify-interface-nondet" root model))
-         (nondets (result-split nondets))
-         (lts-tags (get-tags asserts))
+(define* (verify:interface-asserts model output #:key keep-going?)
+  (let* ((result (result-split output))
+         (lts-tags (get-tags result))
          (unreachable (assert-unreachable lts-tags (model-tags model)))
          (result `(,unreachable
-                   ,@asserts
-                   ,@nondets))
+                   ,@result))
          (deadlock? (get-trace 'deadlock result)))
-    (define* (report-assert assert #:key skip?)
-      (report assert skip? (get-trace assert result) model))
-    (reduce-or (command-line:get 'all)
+    (define* (report-assert assert #:key skip? (trace (get-trace assert result)))
+      (report model assert trace #:skip? skip?))
+    (reduce-or keep-going?
                `(,(cute report-assert 'deadlock)
                  ,@(if (%no-unreachable?) '()
                        `(,(cut report-assert 'unreachable #:skip? deadlock?)))
-                 ,(cute report-assert 'livelock)
-                 ,(cute report-assert 'deterministic)))))
+                 ,(cute report-assert 'livelock)))))
 
-(define (mcrl2:verify-component-asserts model root)
-  (let* ((model-name (makreel:unticked-dotted-name model))
-         (result status (verify-pipeline (component-stage) root model))
-         (result (result-split result))
+(define* (verify:interface-nondet-assert model output #:key keep-going?)
+  (let* ((result (result-split output))
+         (assert 'deterministic)
+         (trace (get-trace assert result)))
+    (report model assert trace)))
+
+(define* (verify:component-asserts model output #:key component keep-going?)
+  (let* ((result (result-split output))
          (lts-tags (get-tags result))
          (unreachable (assert-unreachable lts-tags (model-tags model)))
          (result `(,unreachable
                    ,@result))
          (illegal? (get-trace 'illegal result))
-         (deadlock? (get-trace 'deadlock result))
-         (refinement-trace interface-accepts component-accepts
-                           (mcrl2:verify-compliance root model)))
-    (define* (report-assert assert #:key skip? trace)
-      (report assert skip? (or trace (get-trace assert result)) model))
-    (define (extend-trace trace accepts)
-      (if accepts (string-append trace (car accepts) "\n")
-          trace))
-    (reduce-or (command-line:get 'all)
+         (deadlock? (get-trace 'deadlock result)))
+    (define* (report-assert assert #:key skip? (trace (get-trace assert result)))
+      (report model assert trace #:skip? skip?))
+    (reduce-or keep-going?
                `(,(cute report-assert 'deterministic)
                  ,(cute report-assert 'illegal)
                  ,(cute report-assert 'deadlock)
                  ,@(if (%no-unreachable?) '()
                        `(,(cut report-assert 'unreachable
                                #:skip? (or illegal? deadlock?))))
-                 ,(cute report-assert 'livelock)
-                 ,(cut report-assert 'compliance
-                       #:skip? (or illegal? deadlock?)
-                       #:trace refinement-trace)))))
+                 ,(cute report-assert 'livelock)))))
 
-(define (mcrl2:verify-interface root model)
-  (mcrl2:verify-interface-asserts model root))
+(define* (verify:compliance-assert model output #:key
+                                   compliance-output
+                                   compliance-status
+                                   keep-going?
+                                   status)
+  (let* ((result (result-split output))
+         (illegal? (get-trace 'illegal result))
+         (deadlock? (get-trace 'deadlock result))
+         (trace interface-accepts component-accepts
+                (mcrl2:compliance-output->trace compliance-output
+                                                compliance-status)))
+    (define* (report-assert assert #:key skip?)
+      (report model assert trace #:skip? skip?))
+    (reduce-or keep-going?
+               (list (cut report-assert 'compliance
+                          #:skip? (or illegal? deadlock?))))))
 
-(define (mcrl2:verify-component root model)
-  (let ((component model))
-    (reduce-or (command-line:get 'all)
-               (list (cut mcrl2:verify-component-asserts component root)))))
+(define* (mcrl2:verify-interface root model #:key keep-going?)
+  (reduce-or
+   keep-going?
+   (list
+    (lambda _
+      (let ((output status (verify-pipeline "verify-interface" root model)))
+        (verify:interface-asserts model output #:keep-going? keep-going?)))
+    (lambda _
+      (let ((output status (verify-pipeline "verify-interface-nondet" root model)))
+        (verify:interface-nondet-assert model output #:keep-going? keep-going?))))))
 
-(define (mcrl2:verify root model-name)
+(define* (mcrl2:verify-component root model #:key keep-going?)
+  (let ((output status (verify-pipeline "verify-component" root model)))
+    (reduce-or
+     keep-going?
+     (list
+      (cut verify:component-asserts model output #:keep-going? keep-going?)
+      (lambda _
+        (let ((compliance-output
+               compliance-status
+               (verify-pipeline "verify-compliance" root model)))
+          (verify:compliance-assert model output
+                                    ;; XXX FIXME
+                                    #:compliance-output compliance-output
+                                    #:compliance-status compliance-status
+                                    #:keep-going? keep-going?
+                                    #:status status)))))))
+
+(define* (mcrl2:verify root model-name #:key keep-going?)
   (let ((model (makreel:get-model root model-name)))
-    (cond ((is-a? model <interface>) (mcrl2:verify-interface root model))
-          ((is-a? model <component>) (mcrl2:verify-component root model))
-          (else #f))))
+    (cond ((is-a? model <interface>)
+           (mcrl2:verify-interface root model #:keep-going? keep-going?))
+          ((is-a? model <component>)
+           (mcrl2:verify-component root model #:keep-going? keep-going?))
+          (else
+           #f))))
+
+(define-method (makreel:name-equal? (model <model>) name)
+  (equal? name (makreel:unticked-dotted-name model)))
+
+(define* (verification:models root #:key model-name
+                              multiple?
+                              no-interfaces?)
+  (let* ((models (ast:model** root))
+         (model (find (cute makreel:name-equal? <> model-name) models)))
+    (if (is-a? model <interface>) (list model)
+        (let* ((components (if (is-a? model <component>) (list model)
+                               (filter (conjoin (is? <component>)
+                                                (negate ast:imported?)
+                                                .behavior)
+                                       models)))
+               (interfaces (filter (is? <interface>) models))
+               (ports (append-map ast:port* components))
+               (component-interfaces (map .type ports))
+               (interfaces (cond (no-interfaces? '())
+                                 (model component-interfaces)
+                                 ((not multiple?)
+                                  (append interfaces component-interfaces))
+                                 (multiple?
+                                  (filter (negate ast:imported?) interfaces))))
+               (interfaces (delete-duplicates interfaces ast:eq?)))
+          (append interfaces components)))))
 
 
 ;;;
@@ -768,26 +825,18 @@ init for MODEL unless INIT."
   (let ((model (makreel:get-model root model-name)))
     (unmemoized-verify-pipeline out root model #:stdout? #t)))
 
-(define* (verification:verify options root #:key all? model-name
-                              no-interfaces?)
-  (define (model-names-for-verification root)
-    (let* ((models (ast:model** root))
-           (components (filter (conjoin (is? <component>) (negate ast:imported?)
-                                        .behavior) models))
-           (component-names (map makreel:unticked-dotted-name components))
-           (interfaces (filter (is? <interface>) models))
-           (interface-names (map makreel:unticked-dotted-name interfaces))
-           (component-interface-names
-            (map (compose makreel:unticked-dotted-name .type)
-                 (append-map ast:port* components)))
-           (interface-names (append interface-names component-interface-names))
-           (interface-names (delete-duplicates interface-names)))
-      (if no-interfaces? component-names
-          (append interface-names component-names))))
-  (let ((model-names (model-names-for-verification root)))
-    (let loop ((model-names model-names) (error? #f))
-      (if (or (and (not all?) error?) (null? model-names)) (if error? 1 0)
-          (let* ((model-name (car model-names))
-                 (this-error? (mcrl2:verify root model-name))
+(define* (verification:verify options root #:key keep-going? model-name
+                              multiple? no-interfaces?)
+  (let ((models (verification:models
+                 root
+                 #:model-name model-name
+                 #:multiple? multiple?
+                 #:no-interfaces? no-interfaces?)))
+    (let loop ((models models) (error? #f))
+      (if (or (and (not keep-going?) error?) (null? models)) (if error? 1 0)
+          (let* ((model (car models))
+                 (model-name (makreel:unticked-dotted-name model))
+                 (this-error? (mcrl2:verify root model-name
+                                            #:keep-going? keep-going?))
                  (error? (or error? this-error?)))
-            (loop (cdr model-names) error?))))))
+            (loop (cdr models) error?))))))
