@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -25,7 +25,8 @@
 
   #:use-module (ice-9 match)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
+  #:use-module (dzn ast util)
   #:use-module (dzn ast)
   #:use-module (dzn config)
   #:use-module (dzn misc)
@@ -84,7 +85,6 @@
     (begin-step pc (.container r:component-port) component-trigger)))
 
 (define-method (begin-step (pc <program-counter>) (instance <runtime:instance>) (trigger <trigger>))
-  (%debug "* ~s ~s ~a\n" (name instance) (trigger->string trigger) "<begin-step>")
   (define (trigger-runtime-port pc)
     (runtime:port (.instance pc) (.port (.trigger pc))))
   (let* ((pc (push-pc pc trigger instance))
@@ -95,12 +95,14 @@
          (pc (if (and port (ast:provides? port)) (reset-reply pc port-name)
                  pc))
          (r:port (and port (runtime:port instance port))))
+    (unless r:port
+      (throw 'programming-error (format #f "trigger without port ~a" trigger)))
     (cond
      ((and (is-a? instance <runtime:component>)
            (or (get-handling pc instance)
                (assq r:port (.blocked pc)))
            (blocked-port pc instance))
-      (%debug "<blocked-error> collateral ~a ~a\n" (name instance) (trigger->string trigger))
+      (%debug (current-source-location) "<blocked-error> collateral ~a ~a" (name instance) (trigger->string trigger))
       (let ((message (format #f "port `~a' is collaterally blocked"
                              (.name port))))
         (clone pc #:status (make <blocked-error>
@@ -116,7 +118,7 @@
         (or (assq r:port (.blocked pc))
             (and (ast:provides? trigger)
                  (member r:other-port ports ast:equal?))))
-      (%debug "<blocked-error> blocked ~a ~a\n" (name instance) (trigger->string trigger))
+      (%debug (current-source-location) "<blocked-error> blocked ~a ~a" (name instance) (trigger->string trigger))
       (let ((message (format #f "port `~a' is blocked" (.name port))))
         (clone pc #:status (make <blocked-error>
                              #:ast (.statement pc)
@@ -134,7 +136,6 @@
     (begin-step pc r:other-port trigger)))
 
 (define-method (begin-step (pc <program-counter>) (instance <runtime:port>) (trigger <trigger>))
-  (%debug "* ~s ~s ~a\n" (name instance) (trigger->string trigger) "<begin-step>")
   (let* ((pc (push-pc pc trigger instance))
          (port-name (.name (.ast instance)))
          (pc (reset-reply pc port-name)))
@@ -153,8 +154,8 @@
   (step (pop-pc pc))) ;; .previous == #f => RTC
 
 (define-method (step (pc <program-counter>) (o <statement>))
-  (%debug "MISSING: ~a ~a\n" ((compose name .instance) pc) (name o))
-  '())
+  (throw 'programming-error
+         (format #f "statement not implemented: ~a" (ast-name o))))
 
 (define-method (step (pc <program-counter>) (o <defer>))
   (let* ((defer (.defer pc)))
@@ -174,10 +175,11 @@
   (append
    (next-method pc o)
    (if (ast:modeling? (.trigger pc)) '()
-       (list (make-implicit-illegal pc o)))))
+       (let* ((illegal (make-implicit-illegal pc o))
+              (pc (clone pc #:statement illegal)))
+         (list pc)))))
 
 (define-method (step (pc <program-counter>) (o <declarative-compound>))
-  (%debug "  ~s ~s |~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (map (cut clone pc #:statement <>) (ast:statement* o)))
 
 (define (mark-liveness? pc statement)
@@ -192,33 +194,44 @@
        (let ((pc (clone pc #:status (make <end-of-trail>))))
          (list pc))))
 
+(define-method (step (pc <program-counter>) (o <invariant>))
+  '())
+
 (define-method (step (pc <program-counter>) (o <guard>))
-  (%debug "  ~s ~s |~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((statement (.statement o))
          (continuation (continuation pc (list statement)))
          (statement (.statement continuation)))
     (cond ((not (true? (eval-expression pc (.expression o))))
            '())
+          ((and (not (ast:parent o <on>))
+                (null? (tree-collect (is? <on>) o)))
+           '())
           ((mark-liveness? pc statement))
           (else
            (list continuation)))))
 
 (define-method (step (pc <program-counter>) (o <on>))
-  (%debug "  ~s ~s |~a ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o) ((compose trigger->string car ast:trigger*) o))
-  (let ((statement (.statement o)))
-    (cond ((not (find (cute ast:trigger-equal? <> (.trigger pc)) (ast:trigger* o)))
+  (let ((statement (.statement o))
+        (trigger (.trigger pc)))
+    (cond ((not (find (cute ast:trigger-equal? <> trigger) (ast:trigger* o)))
+           (%debug (current-source-location) pc
+                   (format #f "on does not match trigger: ~a"
+                           (trigger->string trigger)))
            '())
           ((mark-liveness? pc statement))
           (else
            (list (clone pc #:statement statement))))))
 
-(define-method (step (pc <program-counter>) (o <illegal>))
+(define-method (step (pc <program-counter>) (o <declarative-illegal>))
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
+  (let ((error (make <implicit-illegal-error> #:message "illegal" #:ast o)))
+    (list (clone pc #:status error))))
+
+(define-method (step (pc <program-counter>) (o <illegal>))
   (let ((error (make <illegal-error> #:message "illegal" #:ast o)))
     (list (clone pc #:status error))))
 
 (define-method (step (pc <program-counter>) (o <compound>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((continuation (continuation pc o))
          (statement (.statement continuation)))
     (cond ((mark-liveness? pc statement))
@@ -226,7 +239,6 @@
            (list continuation)))))
 
 (define-method (step (pc <program-counter>) (o <skip>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((continuation (continuation pc o))
          (statement (.statement continuation)))
     (cond ((mark-liveness? pc statement))
@@ -234,13 +246,11 @@
            (list continuation)))))
 
 (define-method (step (pc <program-counter>) (o <if>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (if (true? (eval-expression pc (.expression o))) (list (continuation pc ((compose list .then) o)))
       (if (.else o) (list (continuation pc ((compose list .else) o)))
           (list (statement-continuation pc o)))))
 
 (define-method (step (pc <program-counter>) (o <call>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((function (.function o))
          (args (map (cute eval-expression pc <>) (ast:argument* o)))
          (continuation-pc (continuation pc o))
@@ -257,12 +267,10 @@
         (list pc))))
 
 (define-method (step (pc <program-counter>) (o <return>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let ((pc (clone pc #:return (eval-expression pc (.expression o)))))
     (list (continuation pc o))))
 
 (define-method (step (pc <program-counter>) (o <action>))
-  (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o) (trigger->string o))
   (cond ((ast:out? o)
          (step-action-up pc o))
         (else
@@ -331,7 +339,8 @@
            (ast:provides? (.ast instance)))
       (list pc))
      ((and (is-a? instance <runtime:port>)
-           (ast:requires? (.ast instance)))
+           (ast:requires? (.ast instance))
+           other-instance)
       (list (enqueue pc o other-instance (q-trigger))))
      ((is-a? other-instance <runtime:component>)
       (list (enqueue pc o other-instance (q-trigger))))
@@ -339,7 +348,6 @@
       (list pc)))))
 
 (define-method (step (pc <program-counter>) (o <assign>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((pc (assign pc (.variable o) (.expression o)))
          (status (.status pc)))
     (if (is-a? status <range-error>)
@@ -347,14 +355,12 @@
         (list (continuation pc o)))))
 
 (define-method (step (pc <program-counter>) (o <variable>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((pc (assign (push-local pc o) o (.expression o)))
          (pc (if (is-a? (.parent o) <compound>) pc
                  (pop-locals pc (list o)))))
     (list (continuation pc o))))
 
 (define-method (step (pc <program-counter>) (o <reply>))
-  (%debug "  ~s ~s ~a => ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o) (.expression o))
   (let* ((instance (.instance pc))
          (port-name (or (.port.name o)
                         (.port.name (.trigger pc))
@@ -366,7 +372,7 @@
                                    #:ast o
                                    #:previous (.parent previous)
                                    #:message "second-reply")))
-                      (%debug "second reply, previous=~a\n" (ast:location->string previous))
+                      (%debug (current-source-location) "second reply, previous=~a" (ast:location->string previous))
                       (clone pc #:status error)))
                    ((is-a? value <void>)
                     (continuation pc o))
@@ -414,12 +420,10 @@
            (list pc)))))
 
 (define-method (step (pc <program-counter>) (o <flush-return>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let ((pc (pop-pc pc)))
     (list pc)))
 
 (define-method (step (pc <program-counter>) (o <q-out>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((q-trigger (.trigger o))
          (instance (.instance pc))
          (pc (pop-pc pc))
@@ -444,11 +448,9 @@
         (list pc))))))
 
 (define-method (step (pc <program-counter>) (o <blocking>))
-  (%debug "  ~s ~s |~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (list (continuation pc o)))
 
 (define-method (step (pc <program-counter>) (o <block>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (if (not (q-empty? pc)) (let ((pc (set-handling! pc)))
                             (list (flush pc)))
       (let ((pc (continuation pc o))
@@ -473,7 +475,6 @@
   (list (continuation pc o)))
 
 (define-method (step (pc <program-counter>) (o <end-of-on>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((deferred (pop-deferred pc))
          (pc (reset-handling! pc))
          (trigger (.trigger pc))
@@ -533,7 +534,6 @@
                   (value (get-reply pc port-name)))
              (and typed? (not value))))))
 
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (cond
    ((typed-reply? pc o)
     (let* ((trigger (.trigger pc))
@@ -541,7 +541,7 @@
            (typed? (ast:typed? type))
            (error (make <missing-reply-error> #:ast o #:type type
                         #:message "missing-reply")))
-      (%debug "missing reply\n")
+      (%debug (current-source-location) "missing reply")
       (list (clone pc #:status error))))
    (else
     (let* ((trigger (.trigger pc))
@@ -554,26 +554,24 @@
            (port (.port o))
            (instance (.instance pc))
            (value (and port (get-reply pc (.name port))))
-           (pc (cond
-                ((not blocking?)
-                 pc)
-                (else
-                 (let* ((r:port (runtime:port instance port))
-                        (blocked (.blocked pc))
-                        (blocked? (assq-ref blocked r:port))
-                        (blocked (if (not blocked?) blocked
-                                     (alist-delete r:port blocked)))
-                        (collateral (.collateral pc))
-                        (collateral-released (.collateral-released pc))
-                        (collateral-released
-                         (if (not (assq-ref collateral r:port)) collateral-released
-                             (append collateral-released (list r:port))))
-                        (released (.released pc))
-                        (released (delete r:port released)))
-                   (clone pc
-                          #:blocked blocked
-                          #:released released
-                          #:collateral-released collateral-released)))))
+           (pc (if (not blocking?) pc
+                   (let* ((r:port (runtime:port instance port))
+                          (blocked (.blocked pc))
+                          (blocked? (assq-ref blocked r:port))
+                          (blocked (if (not blocked?) blocked
+                                       (alist-delete r:port blocked)))
+                          (collateral (.collateral pc))
+                          (collateral-released (.collateral-released pc))
+                          (collateral? (assq-ref collateral r:port))
+                          (collateral-released
+                           (if (not collateral?) collateral-released
+                               (append collateral-released (list r:port))))
+                          (released (.released pc))
+                          (released (delete r:port released)))
+                     (clone pc
+                            #:blocked blocked
+                            #:released released
+                            #:collateral-released collateral-released))))
            (reset-blocked-on-boundary?
             (and (blocked-on-boundary? pc)
                  (eq? instance (blocked-on-boundary? pc))))
@@ -626,7 +624,7 @@
                (statement-continuation pc p)))))
       (($ <if>)
        (statement-continuation pc p))
-      (($ <function>)
+      ((? (is? <function>))
        (pop-pc pc))
       (($ <defer>)
        (clone pc #:statement #f))
@@ -659,7 +657,7 @@
        (let* ((statements (member o (reverse (ast:statement* parent)) ast:eq?))
               (pc (pop-locals pc (filter (is? <variable>) statements))))
          (function-return pc parent)))
-      (($ <function>)
+      ((? (is? <function>))
        (let* ((formals (ast:formal* parent))
               (pc (pop-locals pc formals)))
          (pop-pc pc)))
