@@ -43,7 +43,7 @@
 (define-public dezyne
   (package
     (name "dezyne")
-    (version #!dezyne!# "2.19.0")
+    (version #!dezyne!# "2.19.1")
     (source
      (origin
        (method url-fetch)
@@ -52,7 +52,7 @@
                            "dezyne.org/download/dezyne/"
                            name "-" version ".tar.gz"))
        (sha256
-        (base32 #!dezyne!# "1jkk8zg1f28gfs9sgw1libac1kbk5hdwx5ny284qrd5g2j0anhxa"))))
+        (base32 #!dezyne!# "17gpzmkafql2d3kj2yx293hgrh3l6zhjk63wcirqr7xsxjhava5n"))))
     (build-system gnu-build-system)
     (native-inputs (list guile-3.0 pkg-config))
     (inputs (list bash-minimal
